<template>
  <div 
    @mouseenter="handleMouseEnter"
    @mouseleave="handleMouseLeave">
    <MenuReaction 
      v-if="isShowReaction && showEmoji && status !== 'failed'" 
      :id="id" 
      :is-seller="isSeller" 
      @onSuccess="handleReactionSuccess" 
      @onActive="handleOnActive"
      @mouseenter="handleMouseEnterMenu"
      @mouseleave="handleMouseLeaveMenu"
    />
    <ChatRepliedMessage v-if="reply_from_id && object?.replied_message && status !== 'loading'" :replied-message="object?.replied_message" class="cursor-pointer" @click="viewReply()"/>
    <ReactionIcon :reaction-list="reactionArr"/>
    <span v-if="general.chatTab != 'email'" v-html="message" class="overflow-hidden relative block break-words"></span>
    <div v-else>
      <div v-if="replyMessage?.length == 0">
        <span v-html="message" class="overflow-hidden relative block break-words"></span>
      </div>
      <div v-else>
        <span v-html="aboveMessage" class="overflow-hidden relative block break-words"></span>
        <div class="relative flex flex-row">
          <div class="mb-5">
            <a-collapse 
              accordion 
              :bordered="false" 
              style="background-color: transparent; box-shadow: none; padding: 0; border-bottom: none !important;"
            >
              <a-collapse-panel 
                key="1" 
                :showArrow="false" 
                style="background-color: transparent; box-shadow: none; padding: 0; margin: 0; border-bottom: none !important;"
              >
                <template #header>
                  <div 
                    style="display: flex; align-items: left; justify-content: left; padding: 0; width: 100%;"
                  >
                    <button 
                      style="
                        background-color: #ddd;
                        border: none;
                        color: black;
                        padding: 2px 10px;
                        margin: 0;
                        border-radius: 16px;
                        font-size: 12px;
                        line-height: 1;
                        cursor: pointer;
                      "
                    >
                      ...
                    </button>
                  </div>
                </template>
                <span v-html="replyMessage"></span>
              </a-collapse-panel>
            </a-collapse>
          </div>
        </div>
      </div>
    </div>
    
    <Attachments v-if="object?.attachment?.length > 0" :object="object"/>
      
    <FailedSendMessage v-if="isSeller && status ==='failed'" :message="wa_error_message"/>

    <MenuChatBubble v-if="isShowMenuChat" :is-seller="isSeller" :message="message" :id="id" :chat_type="chatType"/>

    <div v-if="is_forwarded === 1"
          class="absolute  -top-8 gap-1 rounded-tl-xl rounded-tr-xl left-0 block w-full  ">
      <div class="relative flex py-2 items-center">
        <svg xmlns="http://www.w3.org/2000/svg" class="w-5 h-5 fill-current text-gray-400"
              viewBox="0 0 24 24">
          <path d="M21,12L14,5V9C7,10 4,15 3,20C5.5,16.5 9,14.9 14,14.9V19L21,12Z"/>
        </svg>
        <span class="italic text-gray-400">Forwarded</span>
      </div>
    </div>
    <div :class="`relative flex items-end gap-5 ${assignee ? 'justify-between' : 'justify-end'}` ">
      <div v-if="isFromApplication && isSeller" class="flex items-center gap-2 whitespace-nowrap  rounded-full py-1 px-2 bottom-1 text-xs bg-white text-primary text-right select-none">
        <span class="w-2 h-2 bg-scheme-primary rounded-full"></span>
        <span>{{ applicationAgentName }}</span>
      </div>
      <div v-else-if="assignee"
            class="flex items-center gap-2 whitespace-nowrap  rounded-full py-1 px-2 bottom-1 text-xs bg-white text-primary text-right select-none">
        <span class="w-2 h-2 bg-scheme-primary rounded-full"></span>
        <span>{{ assignee.full_name }}</span>
      </div>
      <div class="flex gap-1 items-center justify-end">
        <span class="block  text-xs text-gray-600 text-right select-none">{{ date }}</span>
        <!-- loading -->
        <template v-if="isSeller && !isLoader">
          <IcMessageStatus :status="status"/>
        </template>
      </div>
    </div>
  </div>
  <!--  <div-->
  <!--      :class="`relative flex gap-4 w-full ${isLoader ? 'animate-pulse': ''}  ${isSeller ? 'flex-row-reverse' : 'flex-row'}`">-->

  <!--    <div-->
  <!--        :class="`relative overflow-hidden w-14 h-14 object-cover  ${isLoader ? 'bg-gray-400': 'bg-white shadow-md'}  rounded-full flex items-center ${isSeller ? 'order-2 p-3 ' : 'p-3 order-1'}`">-->
  <!--      <div v-if="!isLoader" class="relative block h-14 w-14 overflow-hidden object-cover">-->
  <!--        <ic-person-general />-->
  <!--      </div>-->
  <!--      &lt;!&ndash;        <Avatar :url="'https://source.unsplash.com/random/150x150/?avatar'"/>&ndash;&gt;-->
  <!--    </div>-->
  <!--    <div-->
  <!--        :class="`relative min-w-200 max-w-bubble w-auto mb-12 shadow-sm py-2 pl-2 pr-6 ${isSeller ? 'chat-right order-2 bg-scheme-primary bg-opacity-20' : 'chat-left order-1 bg-gray-100'} ${isLoader ? 'bg-gray-300': ''}`">-->
  <!--      <div class="relative inline-block mb-2 overflow-hidden text-base whitespace-normal pb-2 text-gray-800"-->
  <!--           v-html="message">-->
  <!--      </div>-->


  <!--      <FailedSendMessage v-if="isSeller && isError"/>-->
  <!--    </div>-->
  <!--  </div>-->
</template>

<script setup lang="ts">
import FailedSendMessage from "../Utility/FailedSendMessage.vue";
import IcPersonGeneral from "../Icon/IcPersonGeneral.vue";
import IcMessageStatus from "../Icon/IcMessageStatus.vue";
import {onBeforeUnmount, onMounted, ref, watch} from "vue";
import MenuChatBubble from "../Menu/MenuChatBubble.vue";
import helpers from "../../utils/helpers";
import ChatRepliedMessage from "./ChatRepliedMessage.vue";
import ReactionIcon from "../Icon/ReactionIcon.vue";
import MenuReaction from "../../components/Menu/MenuReaction.vue";
import {useChatListStore} from "../../stores/chat_list";
import {useGeneralStore} from "../../stores/general";
import Attachments from "../Utility/Attachments.vue";
import {useArrayFind, useArrayFindIndex} from "@vueuse/core";

interface Props {
  isSeller?: boolean
  isError?: boolean
  isLoading?: boolean
  isShowReaction?: boolean
  isShowMenuChat?: boolean
  status?: string
  isLoader?: boolean
  id?: bigint
  date?: string
  is_forwarded?: number
  reply_from_id?: string | number
  message?: string
  wa_error_message?: string
  chatType?: string
  isFromApplication: boolean | null
  applicationAgentName: string | null
  assignee?: {
    full_name?: string
  }
  object?: any
  reactions?: any
}

let props = withDefaults(defineProps<Props>(), {
  isSeller: false,
  isError: false,
  isShowReaction: true,
  isShowMenuChat: true,
  is_forwarded: 0,
  isLoader: false,
  isLoading: false,
  date: null,
  id: null,
  status: null,
  assignee: null,
  isFromApplication: null,
  applicationAgentName: null,
  chatType: null,
  message: null,
  wa_error_message: null,
  reply_from_id: null,
  reactions: null,
  object: {
    replied_message: {
      message: null,
      name: null,
      type: null,
      object: null,
    }
  },
});

const chat = useChatListStore()
const general = useGeneralStore()
const showEmoji = ref(false)
const emojiActive = ref(false)
const reactionArr = ref([])
const aboveMessage = ref('')
const replyMessage = ref('')
const isHoveringMenu = ref(false)
const leaveTimeout = ref()

const handleMouseEnter = () => {
  if (leaveTimeout.value) {
    clearTimeout(leaveTimeout.value);
    leaveTimeout.value = null;
  }
  showEmoji.value = true;
};

const handleMouseEnterMenu = () => {
  isHoveringMenu.value = true;
  handleMouseEnter();
};

const handleMouseLeaveMenu = () => {
  isHoveringMenu.value = false;
  startHideTimeout();
};

const handleMouseLeave = () => {
  startHideTimeout();
};

const startHideTimeout = () => {
  leaveTimeout.value = setTimeout(() => {
    if (!emojiActive.value && !isHoveringMenu.value) {
      showEmoji.value = false;
    }
  }, 200);
};

onBeforeUnmount(() => {
  if (leaveTimeout.value) {
    clearTimeout(leaveTimeout.value);
  }
});

const formatEmail = (message: string) => {
  if(general.chatTab != 'email' || typeof message != 'string'){
    return message
  }
  
  if (message.includes('__thread_separator__')) {
      if (message.includes('Reply:<br>')) {
          message = message.replaceAll('__thread_separator__', '');
      } else {
          message = message.replaceAll('__thread_separator__', 'Reply:<br>');
      }
  }
  
  const firstReplyIndex = message.indexOf("Reply:<br>");

  if (firstReplyIndex !== -1) {
    const beforeFirstReply = message.slice(0, firstReplyIndex);
    const restOfMessage = message.slice(firstReplyIndex);

    aboveMessage.value = beforeFirstReply.replace(/<br>+$/, "");
    replyMessage.value = restOfMessage
  }
  else{
    aboveMessage.value = message
  }

  return true
}

const handleReactionSuccess = (emojiString: any) => {
  let type_chat = general.chatTab
  let platform = ''

  if(general.chatTab === 'instagram'){
    type_chat = 'instagram'

    if(chat.selectedChatType === 'instagram'){
      platform = 'instagram'
    }
    
    if(chat.selectedChatType === 'facebook'){
      platform = 'page'
    }
  }

  chat.sendEmoji(chat.selectedChat, {
    to: chat?.detail?.customer?.phone,
    chat_id: props.id,
    emoji: emojiString,
    platform: platform
  },type_chat).then((resp) => {
    if (resp.code === 200) {
      const currentReaction = reactionArr.value.findIndex((rc) => rc.chat_id === props.id && rc.from_chat_user_id === chat.selectedChat)
      if (currentReaction == -1 && reactionArr.value.length == 0) {
        reactionArr.value.push({
          chat_id : props?.id,
          from_chat_user_id: chat.selectedChat,
          value: emojiString
        })
      } else {

        reactionArr.value.splice(currentReaction,1, {
          chat_id : props?.id,
          from_chat_user_id: chat.selectedChat,
          value: emojiString
        })
      }
    } else {
      console.log("400")
    }
  })
}

const viewReply= () => {
  general.isShowReplyChat = !general.isShowReplyChat

  if(general.isShowReplyChat == true){
    general.isShowProfile = false
    general.replyChatId = props?.id
  }else{
    general.replyChatId = null
  }
}

const customHeader = () =>{
  return `<div style="display: flex; align-items: center; justify-content: space-between;">
          <span>View Replies</span>
          <span style="cursor: pointer;">...</span>
        </div>`;
}

const handleOnActive= (event) => {
  emojiActive.value = event
}

onMounted(() => {
  reactionArr.value = props.reactions
  formatEmail(props.message)
})

watch(() => props.reactions, (val) => {
  reactionArr.value = val
})

</script>
